/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2018-2023. All rights reserved.
 */

package com.huawei.apig.sdk.demo;

import com.huawei.apig.sdk.util.Constant;
import com.huawei.apig.sdk.util.HostName;
import com.huawei.apig.sdk.util.SSLCipherSuiteUtil;

import com.cloud.apigateway.sdk.utils.Client;
import com.cloud.apigateway.sdk.utils.Request;

import okhttp3.OkHttpClient;
import okhttp3.Response;
import okhttp3.WebSocket;
import okhttp3.WebSocketListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class WebSocketDemo {
    private static final Logger LOGGER = LoggerFactory.getLogger(WebSocketDemo.class);

    public static void main(String[] args) throws Exception {
        // Create a new request.
        Request webSocketRequest = new Request();
        try {
            // 认证用的ak和sk硬编码到代码中或者明文存储都有很大的安全风险，建议在配置文件或者环境变量中密文存放，使用时解密，确保安全；
            // 本示例以ak和sk保存在环境变量中为例，运行本示例前请先在本地环境中设置环境变量HUAWEICLOUD_SDK_AK和HUAWEICLOUD_SDK_SK。
            webSocketRequest.setKey(System.getenv("HUAWEICLOUD_SDK_AK"));
            webSocketRequest.setSecret(System.getenv("HUAWEICLOUD_SDK_SK"));
            webSocketRequest.setMethod("GET");
            webSocketRequest.setUrl("your url");
        } catch (Exception e) {
            LOGGER.error("fail to contain request: {}", e.getMessage());
            throw e;
        }
        try {
            // Sign the request.
            okhttp3.Request signedRequest = Client.signOkhttp(webSocketRequest, Constant.SIGNATURE_ALGORITHM_SDK_HMAC_SHA256);
            OkHttpClient webSocketClient;
            if (Constant.DO_VERIFY) {
                // creat okhttpClient and verify ssl certificate
                HostName.setUrlHostName(webSocketRequest.getHost());
                webSocketClient = SSLCipherSuiteUtil.createOkHttpClientWithVerify(Constant.INTERNATIONAL_PROTOCOL);
            } else {
                // creat okhttpClient and do not verify ssl certificate
                webSocketClient = SSLCipherSuiteUtil.createOkHttpClient(Constant.INTERNATIONAL_PROTOCOL);
            }
            WebSocketListener webSocketListener = new WebSocketListener() {
                @Override
                public void onMessage(WebSocket webSocket, String text) {
                    LOGGER.info("receive: " + text);
                }

                @Override
                public void onFailure(WebSocket webSocket, Throwable t, Response response) {
                    LOGGER.info("fail   : " + t.getMessage());
                }
            };
            WebSocket webSocket = webSocketClient.newWebSocket(signedRequest, webSocketListener);
            for (int i = 0; i < 10; i++) {
                String msg = "hello," + System.currentTimeMillis();
                LOGGER.info("send   : " + msg);
                webSocket.send(msg);
                Thread.sleep(50);
            }
            webSocket.close(1000, null);
            webSocketClient.dispatcher().executorService().shutdown();
        } catch (Exception e) {
            LOGGER.error("fail to send the request: {}", e.getMessage());
            throw e;
        }
    }
}